using System;
using System.Collections.Generic;
using System.Data;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.HtmlControls;

namespace SelectorRadioButton.Controls
{
    /// <summary>
    ///     Represents a field that is displayed as a radio button in a data-bound 
    ///     <see cref="System.Web.UI.WebControls.GridView">GridView</see> control and is
    ///     used to select a record in the GridView. 
    /// </summary>
    /// <remarks>
    ///     <para>
    ///         The <strong>SelectorRadioButtonField</strong> class is used by 
    ///         <see cref="System.Web.UI.WebControls.GridView">GridView</see> controls that are bound
    ///         to a <see cref="System.Data.DataTable">DataTable</see> with a single column primary key. 
    ///         The <strong>SelectorRadioButtonField</strong> will be bound to the primary key column of the 
    ///         DataTable. The <strong>SelectorRadioButtonField</strong> class is used to display a column of 
    ///         radio buttons. The <strong>value</strong> attribute of the radio button for each row in the 
    ///         GridView will be the value of the primary key column of the row.
    ///     </para>
    ///     <para>
    ///         To add a <strong>SelectorRadioButtonField</strong> object to a <strong>GridView</strong> 
    ///         control, create a new <strong>SelectorRadioButtonField</strong> object, set its properties, 
    ///         and then add it to the <strong>GridView</strong> control's 
    ///         <see cref="System.Web.UI.WebControls.GridView.Columns">Columns</see> collection. You must
    ///         set at least the <see cref="Name">Name</see> and <see cref="ValueDataField">ValueDataField</see>
    ///         properties. 
    ///     </para>
    /// </remarks>
    public class SelectorRadioButtonField : TemplateField
    {
        #region Private Instance Variables
        //----------------------------------------------------------------------------------------------------
        private SelectorRadioButtonItem _selectorRadioButtonItem;
        //----------------------------------------------------------------------------------------------------
        #endregion

        #region Public Properties
        //----------------------------------------------------------------------------------------------------
        /// <summary>
        ///     Gets or sets the value of the <strong>name</strong> attribute of the HTML <![CDATA[<input>]]> 
        ///     element that is rendered for the radio buttons.
        /// </summary>
        public string Name
        {
            get { return _selectorRadioButtonItem.Name; }
            set { _selectorRadioButtonItem.Name = value; }
        }

        /// <summary>
        ///     Gets or sets the name of the column in the bound DataTable that will provide the values of the
        ///     <strong>value</strong> attribute of the HTML <![CDATA[<input>]]> element that is rendered for the 
        ///     radio buttons. This should be the name of the primary key column of the DataTable.  
        /// </summary>
        public string ValueDataField
        {
            get { return _selectorRadioButtonItem.ValueDataField; }
            set { _selectorRadioButtonItem.ValueDataField = value; }
        }

        /// <summary>
        ///     Gets or sets the value of the <strong>onclick</strong> attribute of the HTML <![CDATA[<input>]]> 
        ///     element that is rendered for the radio buttons.
        /// </summary>
        public string OnClick
        {
            get { return _selectorRadioButtonItem.OnClick; }
            set { _selectorRadioButtonItem.OnClick = value; }
        }

        /// <summary>
        ///     Gets a <see cref="AttributeBindingsCollection">AttributeBindingsCollection</see> object of 
        ///     attribute name and bound data field pairs.
        /// </summary>
        /// <remarks>
        ///     <para>
        ///         Use this property to add data bound attributes to the HTML <![CDATA[<input>]]> element  
        ///         that is rendered for the radio button.
        ///     </para>
        ///     <para>
        ///         For additional information on the attribute bindings collection, see the 
        ///         <see cref="AttributeBindingsCollection">AttributeBindingsCollection</see> class.
        ///     </para>
        /// </remarks>
        public AttributeBindingsCollection AttributeBindings
        {
            get { return _selectorRadioButtonItem.AttributeBindings; }
        }

        /// <summary>
        ///     Gets a <see cref="UnBoundAttributesCollection">UnBoundAttributesCollection</see> object of 
        ///     attribute name and value pairs.
        /// </summary>
        /// <remarks>
        ///     <para>
        ///         Use this property to add attributes to the HTML <![CDATA[<input>]]> element  
        ///         that is rendered for the radio button, where the same value will be used for each 
        ///         radio button that is rendered in the column of the GridView (i.e. the value will 
        ///         not be bound to a column in the bound DataTable.
        ///     </para>
        ///     <para>
        ///         For additional information on the unbound attributes collection, see the 
        ///         <see cref="UnBoundAttributesCollection">UnBoundAttributesCollection</see> class.
        ///     </para>
        /// </remarks>
        public UnBoundAttributesCollection UnBoundAttributes
        {
            get { return _selectorRadioButtonItem.UnBoundAttributes; }
        }

        /// <summary>
        ///     Do not access the ItemTemplate property for SelectorRadioButtonField. 
        /// </summary>
        public override ITemplate ItemTemplate
        {
            get
            {
                throw new InvalidOperationException(
                    "Do not access the ItemTemplate property for SelectorRadioButtonField.");
            }
            set
            {
                throw new InvalidOperationException(
                    "Do not set the ItemTemplate property for SelectorRadioButtonField.");
            }
        }
        //----------------------------------------------------------------------------------------------------
        #endregion

        #region Constructors
        //----------------------------------------------------------------------------------------------------
        /// <summary>
        ///     Initializes a new instance of the 
        ///     <see cref="SelectorRadioButtonField">SelectorRadioButtonField</see> class. 
        /// </summary>
        /// <remarks>
        ///     <para>
        ///         Use this constructor to initialize a new instance of the  
        ///         <strong>SelectorRadioButtonField</strong> class. This constructor is used 
        ///         when adding a column of selector radio buttons to a 
        ///         <see cref="System.Web.UI.WebControls.GridView">GridView</see> control.
        ///     </para>
        ///     <para>
        ///         To add a <strong>SelectorRadioButtonField</strong> object to a <strong>GridView</strong> 
        ///         control, create a new <strong>SelectorRadioButtonField</strong> object, set its properties, 
        ///         and then add it to the <strong>GridView</strong> control's 
        ///         <see cref="System.Web.UI.WebControls.GridView.Columns">Columns</see> collection. 
        ///     </para>
        /// </remarks>
        public SelectorRadioButtonField()
        {
            _selectorRadioButtonItem = new SelectorRadioButtonItem();
            base.ItemTemplate = _selectorRadioButtonItem;

            ItemStyle.HorizontalAlign = HorizontalAlign.Center;
            ItemStyle.Width = new Unit(40, UnitType.Pixel);
        }

        /// <summary>
        ///     Initializes a new instance of the 
        ///     <see cref="SelectorRadioButtonField">SelectorRadioButtonField</see> class.
        /// </summary>
        /// <param name="name">
        ///     The value of the <strong>name</strong> attribute of the HTML <![CDATA[<input>]]> 
        ///     element that is rendered for the radio buttons.
        /// </param>
        /// <param name="valueDataField">
        ///     The name of the column in the bound DataTable that will provide the values of the
        ///     <strong>value</strong> attribute of the HTML <![CDATA[<input>]]> element that is rendered for the 
        ///     radio buttons. This should be the name of the primary key column of the DataTable.  
        /// </param>
        /// <remarks>
        ///     <para>
        ///         Use this constructor to initialize a new instance of the  
        ///         <strong>SelectorRadioButtonField</strong> class. This constructor is used 
        ///         when adding a column of selector radio buttons to a 
        ///         <see cref="System.Web.UI.WebControls.GridView">GridView</see> control.
        ///     </para>
        ///     <para>
        ///         To add a <strong>SelectorRadioButtonField</strong> object to a <strong>GridView</strong> 
        ///         control, create a new <strong>SelectorRadioButtonField</strong> object, set its properties, 
        ///         and then add it to the <strong>GridView</strong> control's 
        ///         <see cref="System.Web.UI.WebControls.GridView.Columns">Columns</see> collection. 
        ///     </para>
        /// </remarks>
        public SelectorRadioButtonField(string name, string valueDataField) 
            : this()
        {
            Name = name;
            ValueDataField = valueDataField;
        }
        //----------------------------------------------------------------------------------------------------
        #endregion

        #region Private Classes
        //----------------------------------------------------------------------------------------------------

        #region SelectorRadioButtonItem
        //----------------------------------------------------------------------------------------------------

        private class SelectorRadioButtonItem : ITemplate
        {
            #region Private Instance Variables
            //----------------------------------------------------------------------------------------------------
            private string _name = null;
            private string _valueDataField = null;
            private string _onClick = null;
            private AttributeBindingsCollection _attributeBindings = new AttributeBindingsCollection();
            private UnBoundAttributesCollection _unBoundAttributes = new UnBoundAttributesCollection();
            //----------------------------------------------------------------------------------------------------
            #endregion

            #region Public Properties
            //----------------------------------------------------------------------------------------------------
            public string Name
            {
                get { return _name; }
                set { _name = value; }
            }

            public string ValueDataField
            {
                get { return _valueDataField; }
                set { _valueDataField = value; }
            }

            public string OnClick
            {
                get { return _onClick; }
                set { _onClick = value; }
            }

            public AttributeBindingsCollection AttributeBindings
            {
                get { return _attributeBindings; }
            }

            public UnBoundAttributesCollection UnBoundAttributes
            {
                get { return _unBoundAttributes; }
            }
            //----------------------------------------------------------------------------------------------------
            #endregion

            #region ITemplate Members
            //----------------------------------------------------------------------------------------------------
            public void InstantiateIn(Control container)
            {
                SelectorRadioButton radioButton = new SelectorRadioButton();

                radioButton.DataBinding += new EventHandler(radioButton_DataBinding);

                if (string.IsNullOrEmpty(_name))
                {
                    throw new InvalidOperationException("The Name property for SelectorRadioButtonField has not been set.");
                }
                else
                {
                    radioButton.Name = _name;
                }

                if (!string.IsNullOrEmpty(_onClick))
                {
                    radioButton.Attributes.Add(HtmlTextWriterAttribute.Onclick.ToString(), _onClick);
                }

                foreach (string key in _unBoundAttributes.Keys)
                {
                    radioButton.Attributes.Add(key, _unBoundAttributes[key]);
                }

                container.Controls.Add(radioButton);
            }
            //----------------------------------------------------------------------------------------------------
            #endregion

            #region Events
            //----------------------------------------------------------------------------------------------------
            private void radioButton_DataBinding(object sender, EventArgs e)
            {
                SelectorRadioButton radioButton = (SelectorRadioButton)sender;
                GridViewRow container = (GridViewRow)radioButton.NamingContainer;
                DataRowView row = (DataRowView)container.DataItem;

                if (string.IsNullOrEmpty(_valueDataField))
                {
                    throw new InvalidOperationException("The ValueDataField property for SelectorRadioButtonField has not been set.");
                }
                else
                {
                    radioButton.Value = row[_valueDataField].ToString();
                }

                foreach (string key in _attributeBindings.Keys)
                {
                    radioButton.Attributes.Add(key, row[_attributeBindings[key]].ToString());
                }
            }
            //----------------------------------------------------------------------------------------------------
            #endregion

            #region Public Static Methods
            //----------------------------------------------------------------------------------------------------
            public static void ValidateAttributeName(string attributeName)
            {
                // Ensure that the attribute name is not one of the 
                // attribute we are providing a customised property

                const string NameAttribute = "name";
                const string ValueAttribute = "value";
                const string OnClickAttribute = "onclick";

                string attributeNameLower = attributeName.ToLower();

                if (attributeNameLower == NameAttribute)
                {
                    throw new ArgumentException("Use the Name property to specify the name attribute");
                }
                else if (attributeNameLower == ValueAttribute)
                {
                    throw new ArgumentException("Use the ValueDataField property to specify the value attribute");
                }
                else if (attributeNameLower == OnClickAttribute)
                {
                    throw new ArgumentException("Use the OnClick property to specify the onclick attribute");
                }
            }
            //----------------------------------------------------------------------------------------------------
            #endregion
        }

        //----------------------------------------------------------------------------------------------------
        #endregion

        #region SelectorRadioButton
        //----------------------------------------------------------------------------------------------------

        private class SelectorRadioButton : HtmlInputRadioButton
        {
            protected override void Render(HtmlTextWriter output)
            {
                foreach (string key in Attributes.Keys)
                {
                    output.AddAttribute(key, Attributes[key]);
                }

                output.RenderBeginTag(HtmlTextWriterTag.Input);
                output.RenderEndTag();
            }
        }

        //----------------------------------------------------------------------------------------------------

        #endregion
        
        //----------------------------------------------------------------------------------------------------
        #endregion

        #region Attribute Collection Classes
        //----------------------------------------------------------------------------------------------------

        #region AttributeBindingsCollection
        //----------------------------------------------------------------------------------------------------
        /// <summary>
        ///     Provides a collection of attribute name and bound data field pairs.
        /// </summary>
        /// <remarks>
        ///     <para>
        ///         The <strong>AttributeBindingsCollection</strong> class is inherited from the
        ///         generic <see cref="System.Collections.Generic.Dictionary">Dictionary</see> class.
        ///     </para>
        ///     <para>
        ///         The key of each value in the <strong>AttributeBindingsCollection</strong> is the 
        ///         name of an attribute that will be included in the HTML <![CDATA[<input>]]> element 
        ///         that is rendered for the radio buttons.
        ///     </para>
        ///     <para>
        ///         A value added to the <strong>AttributeBindingsCollection</strong> is the name of a
        ///         column in the <see cref="System.Data.DataTable">DataTable</see> that will be bound 
        ///         to the <see cref="System.Web.UI.WebControls.GridView">GridView</see> control.  
        ///         The column values will be rendered in the HTML <![CDATA[<input>]]> elements for the 
        ///         radio buttons as the value of the attibute named with the key in the 
        ///         <strong>AttributeBindingsCollection</strong>.
        ///     </para>
        /// </remarks>
        public class AttributeBindingsCollection : Dictionary<string, string>
        {
            /// <summary>
            ///     Adds the specified attribute and dataField to the collection. 
            /// </summary>
            /// <param name="attribute">
            ///     The name of an attribute that will be included in the HTML <![CDATA[<input>]]> 
            ///     element that is rendered for the radio buttons.
            /// </param>
            /// <param name="dataField">
            ///     The name of a column in the <see cref="System.Data.DataTable">DataTable</see> that will 
            ///     be bound to the <see cref="System.Web.UI.WebControls.GridView">GridView</see> control.  
            ///     The column values will be rendered in the HTML <![CDATA[<input>]]> elements for the 
            ///     radio buttons as the value of the attibute named with the key in the 
            ///     <strong>AttributeBindingsCollection</strong>.
            /// </param>
            public new void Add(string attribute, string dataField)
            {
                SelectorRadioButtonItem.ValidateAttributeName(attribute);
                base.Add(attribute, dataField);
            }
        }
        //----------------------------------------------------------------------------------------------------
        #endregion

        #region UnBoundAttributesCollection
        //----------------------------------------------------------------------------------------------------
        /// <summary>
        ///     Provides a collection of attribute name and value pairs.
        /// </summary>
        /// <remarks>
        ///     <para>
        ///         The <strong>UnBoundAttributesCollection</strong> class is inherited from the
        ///         generic <see cref="System.Collections.Generic.Dictionary">Dictionary</see> class.
        ///     </para>
        ///     <para>
        ///         The key of each value in the <strong>UnBoundAttributesCollection</strong> is the 
        ///         name of an attribute that will be included in the HTML <![CDATA[<input>]]> element 
        ///         that is rendered for the radio buttons.
        ///     </para>
        ///     <para>
        ///         A value added to the <strong>UnBoundAttributesCollection</strong> is the value that
        ///         will be rendered in the HTML <![CDATA[<input>]]> elements for the 
        ///         radio buttons as the value of the attibute named with the key in the 
        ///         <strong>UnBoundAttributesCollection</strong>.
        ///     </para>
        /// </remarks>
        public class UnBoundAttributesCollection : Dictionary<string, string>
        {
            /// <summary>
            ///     Adds the specified attribute and value to the collection. 
            /// </summary>
            /// <param name="attribute">
            ///     The name of an attribute that will be included in the HTML <![CDATA[<input>]]> 
            ///     element that is rendered for the radio buttons.
            /// </param>
            /// <param name="value">
            ///     The value that will be rendered in the HTML <![CDATA[<input>]]> elements for the 
            ///     radio buttons as the value of the attibute named with the key in the 
            ///     <strong>UnBoundAttributesCollection</strong>.
            /// </param>
            public new void Add(string attribute, string value)
            {
                SelectorRadioButtonItem.ValidateAttributeName(attribute);
                base.Add(attribute, value);
            }
        }
        //----------------------------------------------------------------------------------------------------
        #endregion

        //----------------------------------------------------------------------------------------------------
        #endregion
    }
}
